import argparse, os
from loguru import logger
from .helpers.sahi import get_slice_bboxes, read_image_as_pil


def slice_image(
    file_path="image.tif",
    save_to_file_prefix="",
    save_to_dir="./output",
    slice_width: int = 512,
    slice_height: int = 512,
    overlap_width_ratio: int = 0.2,
    overlap_height_ratio: int = 0.2,
):

    logger.info("Loading the file {}".format(file_path))

    # Read the image from disk
    image_pil = read_image_as_pil(file_path)

    logger.debug("The image shape is {}".format(image_pil.size))

    image_width, image_height = image_pil.size

    if not (image_width != 0 and image_height != 0):
        raise AssertionError("Invalid image size {}".format(image_pil.size))

    logger.debug("Calculating the slice box positions.")

    slice_bboxes = get_slice_bboxes(
        image_height=image_height,
        image_width=image_width,
        slice_height=slice_height,
        slice_width=slice_width,
        overlap_height_ratio=overlap_height_ratio,
        overlap_width_ratio=overlap_width_ratio,
    )

    os.makedirs(save_to_dir, exist_ok=True)

    part = 0

    # Iterate over the slice bounding boxes
    for slice_bbox in slice_bboxes:
        # Crop the image using the sliced bounding box coordinates
        image_pil_slice = image_pil.crop(slice_bbox)

        # Generate an incrementing file name
        save_file_path = save_to_file_prefix + format(part, "04") + ".jpg"

        # Save the file to disk
        image_pil_slice.save(os.path.join(save_to_dir, save_file_path))

        # Increment the file name
        part += 1

    logger.info("Saved {} image slices to {}".format(part, save_to_dir))


def app():

    parser = argparse.ArgumentParser(description="Slices an image into smaller images.")
    parser.add_argument(
        "-f",
        "--file_path",
        help="The path to a directory of images of path to a file.",
        required=True,
        type=str,
    )
    parser.add_argument(
        "-s",
        "--save_to_dir",
        help="The directory to save the slices to.",
        required=True,
        type=str,
    )
    parser.add_argument(
        "-sw",
        "--slice_width",
        help="The width of each slice.",
        required=True,
        type=int,
    )
    parser.add_argument(
        "-sh",
        "--slice_height",
        help="The height of each slice.",
        required=True,
        type=int,
    )
    parser.add_argument(
        "-ow",
        "--overlap_width_ratio",
        help="The overlap width ratio.",
        default=0.2,
        type=float,
    )
    parser.add_argument(
        "-oh",
        "--overlap_height_ratio",
        help="The overlap height ratio.",
        default=0.2,
        type=float,
    )

    # image-slicer -f ./tests/test.jpg -s ./output -sw 500 -sh 500
    args = vars(parser.parse_args())

    # If the file path is a file
    if os.path.isfile(args["file_path"]):
        file_path = args["file_path"]
        file_prefix = os.path.splitext(os.path.basename(file_path))[0] + "-"
        slice_image(
            file_path=args["file_path"],
            save_to_file_prefix=file_prefix,
            save_to_dir=args["save_to_dir"],
            slice_width=args["slice_width"],
            slice_height=args["slice_height"],
            overlap_width_ratio=args["overlap_width_ratio"],
            overlap_height_ratio=args["overlap_height_ratio"],
        )

        return

    if os.path.isdir(args["file_path"]):

        # Create an iterator to walk through the directory
        for root, dirs, files in os.walk(args["file_path"], topdown=False):

            # For each file in the acquisition directory
            for file_name in files:

                if file_name.endswith((".jpg", ".jpeg", ".png")):

                    file_path = os.path.join(root, file_name)
                    file_prefix = os.path.splitext(os.path.basename(file_path))[0] + "-"
                    slice_image(
                        file_path=file_path,
                        save_to_file_prefix=file_prefix,
                        save_to_dir=args["save_to_dir"],
                        slice_width=args["slice_width"],
                        slice_height=args["slice_height"],
                        overlap_width_ratio=args["overlap_width_ratio"],
                        overlap_height_ratio=args["overlap_height_ratio"],
                    )



from PIL import Image
import requests
import numpy as np
from typing import List, Union


def exif_transpose(image: Image.Image):
    """
    Transpose a PIL image accordingly if it has an EXIF Orientation tag.
    Inplace version of https://github.com/python-pillow/Pillow/blob/master/src/PIL/ImageOps.py exif_transpose()
    :param image: The image to transpose.
    :return: An image.
    """
    exif = image.getexif()
    orientation = exif.get(0x0112, 1)  # default 1
    if orientation > 1:
        method = {
            2: Image.FLIP_LEFT_RIGHT,
            3: Image.ROTATE_180,
            4: Image.FLIP_TOP_BOTTOM,
            5: Image.TRANSPOSE,
            6: Image.ROTATE_270,
            7: Image.TRANSVERSE,
            8: Image.ROTATE_90,
        }.get(orientation)
        if method is not None:
            image = image.transpose(method)
            del exif[0x0112]
            image.info["exif"] = exif.tobytes()
    return image


def read_image_as_pil(
    image: Union[Image.Image, str, np.ndarray], exif_fix: bool = False
):
    """
    Loads an image as PIL.Image.Image.

    Args:
        image : Can be image path or url (str), numpy image (np.ndarray) or PIL.Image
    """
    # https://stackoverflow.com/questions/56174099/how-to-load-images-larger-than-max-image-pixels-with-pil
    Image.MAX_IMAGE_PIXELS = None
    # read image if str image path is provided
    if isinstance(image, str):
        # read image as pil
        try:
            image_pil = Image.open(
                requests.get(image, stream=True).raw
                if str(image).startswith("http")
                else image
            ).convert("RGB")
            if exif_fix:
                image_pil = exif_transpose(image_pil)
        except:  # handle large/tiff image reading
            try:
                import skimage.io
            except ImportError:
                raise ImportError(
                    "Please run 'pip install -U scikit-image imagecodecs' for large image handling."
                )
            image_sk = skimage.io.imread(image).astype(np.uint8)
            if len(image_sk.shape) == 2:  # b&w
                image_pil = Image.fromarray(image_sk, mode="1")
            if image_sk.shape[2] == 4:  # rgba
                image_pil = Image.fromarray(image_sk, mode="RGBA")
            elif image_sk.shape[2] == 3:  # rgb
                image_pil = Image.fromarray(image_sk, mode="RGB")
            else:
                raise TypeError(
                    f"image with shape: {image_sk.shape[3]} is not supported."
                )
    elif isinstance(image, np.ndarray):
        if image.shape[0] < 5:  # image in CHW
            image = image[:, :, ::-1]
        image_pil = Image.fromarray(image)
    elif isinstance(image, Image.Image):
        image_pil = image
    else:
        TypeError("read image with 'pillow' using 'Image.open()'")
    return image_pil


def get_slice_bboxes(
    image_height: int,
    image_width: int,
    slice_height: int = 512,
    slice_width: int = 512,
    overlap_height_ratio: int = 0.2,
    overlap_width_ratio: int = 0.2,
) -> List[List[int]]:
    """Slices `image_pil` in crops.
    Corner values of each slice will be generated using the `slice_height`,
    `slice_width`, `overlap_height_ratio` and `overlap_width_ratio` arguments.

    Args:
        image_height (int): Height of the original image.
        image_width (int): Width of the original image.
        slice_height (int): Height of each slice. Default 512.
        slice_width (int): Width of each slice. Default 512.
        overlap_height_ratio(float): Fractional overlap in height of each
            slice (e.g. an overlap of 0.2 for a slice of size 100 yields an
            overlap of 20 pixels). Default 0.2.
        overlap_width_ratio(float): Fractional overlap in width of each
            slice (e.g. an overlap of 0.2 for a slice of size 100 yields an
            overlap of 20 pixels). Default 0.2.

    Returns:
        List[List[int]]: List of 4 corner coordinates for each N slices.
            [
                [slice_0_left, slice_0_top, slice_0_right, slice_0_bottom],
                ...
                [slice_N_left, slice_N_top, slice_N_right, slice_N_bottom]
            ]
    """
    slice_bboxes = []
    y_max = y_min = 0
    y_overlap = int(overlap_height_ratio * slice_height)
    x_overlap = int(overlap_width_ratio * slice_width)
    while y_max < image_height:
        x_min = x_max = 0
        y_max = y_min + slice_height
        while x_max < image_width:
            x_max = x_min + slice_width
            if y_max > image_height or x_max > image_width:
                xmax = min(image_width, x_max)
                ymax = min(image_height, y_max)
                xmin = max(0, xmax - slice_width)
                ymin = max(0, ymax - slice_height)
                slice_bboxes.append([xmin, ymin, xmax, ymax])
            else:
                slice_bboxes.append([x_min, y_min, x_max, y_max])
            x_min = x_max - x_overlap
        y_min = y_max - y_overlap
    return slice_bboxes



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['echo1_image_slicer', 'echo1_image_slicer.helpers']

package_data = \
{'': ['*']}

install_requires = \
['Pillow>=9.0.1,<10.0.0',
 'loguru>=0.6.0,<0.7.0',
 'numpy>=1.22.2,<2.0.0',
 'requests>=2.28.1,<3.0.0']

entry_points = \
{'console_scripts': ['image-slicer = '
                     'echo1_image_slicer.echo1_image_slicer:app']}

setup_kwargs = {
    'name': 'echo1-image-slicer',
    'version': '0.1.8',
    'description': '',
    'long_description': '# echo1-image-slicer\n\necho1-image-slicer provides a fast way to slice an image into smaller images\n\n## Installation & Use\n\n```shell\n# Install echo1-image-slicer\npip install echo1-image-slicer\n\n# Run the image-slicer\nimage-slicer \\\n    -f ./tests/test.jpg \\\n    -s ./output \\\n    -sp yolo- \\\n    -sw 500 \\\n    -sh 500 \n\n2022-02-18 16:24:08.959 | INFO     | echo1_image_slicer.echo1_image_slicer:slice_image:17 - Loading the file ./tests/test.jpg\n2022-02-18 16:24:09.015 | DEBUG    | echo1_image_slicer.echo1_image_slicer:slice_image:22 - The image shape is (1333, 1333)\n2022-02-18 16:24:09.015 | DEBUG    | echo1_image_slicer.echo1_image_slicer:slice_image:29 - Calculating the slice box positions.\n2022-02-18 16:24:09.068 | INFO     | echo1_image_slicer.echo1_image_slicer:slice_image:58 - Saved 16 image slices to ./output\n```\n\n## image-slicer help\n\n```shell\nusage: image-slicer [-h] -f FILE_NAME [-sp SAVE_TO_FILE_PREFIX] -s SAVE_TO_DIR -sw SLICE_WIDTH -sh SLICE_HEIGHT\n                    [-ow OVERLAP_WIDTH_RATIO] [-oh OVERLAP_HEIGHT_RATIO]\n\nSlices an image into smaller images.\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -f FILE_NAME, --file_name FILE_NAME\n                        The file name to slice.\n  -sp SAVE_TO_FILE_PREFIX, --save_to_file_prefix SAVE_TO_FILE_PREFIX\n                        The prefix for saved slice file names.\n  -s SAVE_TO_DIR, --save_to_dir SAVE_TO_DIR\n                        The directory to save the slices to.\n  -sw SLICE_WIDTH, --slice_width SLICE_WIDTH\n                        The width of each slice.\n  -sh SLICE_HEIGHT, --slice_height SLICE_HEIGHT\n                        The height of each slice.\n  -ow OVERLAP_WIDTH_RATIO, --overlap_width_ratio OVERLAP_WIDTH_RATIO\n                        The overlap width ratio.\n  -oh OVERLAP_HEIGHT_RATIO, --overlap_height_ratio OVERLAP_HEIGHT_RATIO\n                        The overlap height ratio.\n```\n\n## Thanks\n\nPrevious work done by:\n\n* [GitHub - obss/sahi: A lightweight vision library for performing large scale object detection/ instance segmentation.](https://github.com/obss/sahi)\n',
    'author': 'Michael Mohamed',
    'author_email': 'michael.mohamed@echo1.io',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/e1-io/echo1-image-slicer',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
